{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE DataKinds #-}
{-# LANGUAGE TypeSynonymInstances #-}

module IHaskell.Display.Widgets.Box.Box (
-- * The Box widget
Box, 
     -- * Constructor
     mkBox) where

-- To keep `cabal repl` happy when running from the ihaskell repo
import           Prelude

import           Control.Monad (when, join)
import           Data.Aeson
import           Data.HashMap.Strict as HM
import           Data.IORef (newIORef)
import           Data.Text (Text)
import           Data.Vinyl (Rec(..), (<+>))

import           IHaskell.Display
import           IHaskell.Eval.Widgets
import           IHaskell.IPython.Message.UUID as U

import           IHaskell.Display.Widgets.Types
import           IHaskell.Display.Widgets.Common

-- | A 'Box' represents a Box widget from IPython.html.widgets.
type Box = IPythonWidget BoxType

-- | Create a new box
mkBox :: IO Box
mkBox = do
  -- Default properties, with a random uuid
  uuid <- U.random

  let widgetState = WidgetState $ defaultBoxWidget "BoxView"

  stateIO <- newIORef widgetState

  let box = IPythonWidget uuid stateIO
      initData = object ["model_name" .= str "WidgetModel", "widget_class" .= str "IPython.Box"]

  -- Open a comm for this widget, and store it in the kernel state
  widgetSendOpen box initData $ toJSON widgetState

  -- Return the widget
  return box

instance IHaskellDisplay Box where
  display b = do
    widgetSendView b
    return $ Display []

instance IHaskellWidget Box where
  getCommUUID = uuid
